@@ -1,13 +1,27 @@
 """
 SQLite as alternative storage backend for a TableGroup's data.
 
+For the most part, translation of a TableGroup's tableSchema to SQL works as expected:
+- each table is converted to a "CREATE TABLE" statement
+- each column specifies a column in the corresponding "CREATE TABLE" statement
+- foreignKey constraints are added according to the corresponding tableSchema property.
+
+List-valued foreignKeys are supported as follows: For each pair of tables related through a
+list-valued foreign key, an association table is created. To make it possible to distinguish
+multiple list-valued foreign keys between the same two tables, the association table has
+a column `context`, which stores the name of the foreign key column from which a row in the
+assocation table was created.
+
+SQL table and column names can be customized by passing a translator callable when instantiating
+a `Database`.
+
 SQLite support has the following limitations:
 - lists as values (as specified via the separator attribute of a Column) are only supported for
   string types.
 - regex constraints on strings (as specified via a Datatype's format attribute) are not enforced
   by the database.
 """
-from __future__ import unicode_literals
+import pathlib
 import sqlite3
 from contextlib import closing
 from collections import OrderedDict, defaultdict
@@ -17,7 +31,6 @@ from decimal import Decimal
 import attr
 
 from csvw.datatypes import DATATYPES
-from csvw._compat import pathlib
 
 
 def identity(s):
@@ -76,16 +89,6 @@ def select(db, table):
     return cols, list(cu.fetchall())
 
 
-def select_many_to_many(db, table, context):
-    cu = db.execute(
-        "SELECT {0}, group_concat({1}, ' ') FROM {2} WHERE context = '{3}' GROUP BY {0}".format(
-            quoted(table.columns[0].name),
-            quoted(table.columns[1].name),
-            quoted(table.name),
-            context))
-    return {r[0]: r[1].split() for r in cu.fetchall()}
-
-
 @attr.s
 class ColSpec(object):
     """
@@ -174,6 +177,12 @@ class TableSpec(object):
 
     @classmethod
     def from_table_metadata(cls, table):
+        """
+        Create a `TableSpec` from the schema description of a `csvw.metadata.Table`.
+
+        :param table: `csvw.metadata.Table` instance.
+        :return: `TableSpec` instance.
+        """
         spec = cls(name=table.local_name, primary_key=table.tableSchema.primaryKey)
         list_valued = {c.header for c in table.tableSchema.columns if c.separator}
         for fk in table.tableSchema.foreignKeys:
@@ -181,8 +190,13 @@ class TableSpec(object):
             if not fk.reference.schemaReference:
                 if len(fk.columnReference) == 1 and fk.columnReference[0] in list_valued:
                     # List-valued foreign keys are turned into a many-to-many relation!
-                    assert len(fk.reference.columnReference) == 1
-                    assert spec.primary_key and len(spec.primary_key) == 1
+                    assert len(fk.reference.columnReference) == 1, \
+                        'Composite key {0} in table {1} referenced'.format(
+                            fk.reference.columnReference,
+                            fk.reference.resource)
+                    assert spec.primary_key and len(spec.primary_key) == 1, \
+                        'Table {0} with list-valued foreign must have non-composite ' \
+                        'primary key'.format(spec.name)
                     spec.many_to_many[fk.columnReference[0]] = TableSpec.association_table(
                         spec.name,
                         spec.primary_key[0],
@@ -311,6 +325,23 @@ class Database(object):
             self._connection = sqlite3.connect(':memory:')
         return self._connection
 
+    def select_many_to_many(self, db, table, context):
+        if context is not None:
+            context_sql = "WHERE context = '{0}'".format(context)
+        else:
+            context_sql = ''
+        sql = """\
+SELECT {0}, group_concat({1}, ' '), group_concat(COALESCE(context, ''), '||')
+FROM {2} {3} GROUP BY {0}""".format(
+                quoted(self.translate(table.name, table.columns[0].name)),
+                quoted(self.translate(table.name, table.columns[1].name)),
+                quoted(self.translate(table.name)),
+                context_sql)
+        cu = db.execute(sql)
+        return {
+            r[0]: [(k, v) if context is None else k
+                   for k, v in zip(r[1].split(), r[2].split('||'))] for r in cu.fetchall()}
+
     def read(self):
         res = defaultdict(list)
         with self.connection() as conn:
@@ -335,8 +366,8 @@ class Database(object):
 
                 # Retrieve the many-to-many relations:
                 for col, at in table.many_to_many.items():
-                    for pk, v in select_many_to_many(conn, at, col).items():
-                        refs[pk][col] = v
+                    for pk, v in self.select_many_to_many(conn, at, col).items():
+                        refs[pk][self.translate(tname, col)] = v
 
                 cols, rows = select(conn, self.translate(tname))
                 for row in rows:
@@ -356,17 +387,40 @@ class Database(object):
                     res[self.translate(tname)].append(d)
         return res
 
-    def write_from_tg(self, _force=False, _exists_ok=False):
-        return self.write(_force=_force, _exists_ok=_exists_ok, **self.tg.read())
+    def association_table_context(self, table, column, fkey):
+        """
+        Context for association tables is created calling this method.
 
-    def write(self, _force=False, _exists_ok=False, **items):
+        Note: If a custom value for the `context` column is created by overwriting this method,
+        `select_many_to_many` must be adapted accordingly, to make sure the custom
+        context is retrieved when reading the data from the db.
+
+        :param table:
+        :param column:
+        :param fkey:
+        :return: a pair (foreign key, context)
+        """
+        # The default implementation takes the column name as context:
+        return fkey, column
+
+    def write_from_tg(self, _force=False, _exists_ok=False, _skip_extra=False):
+        return self.write(
+            force=_force,
+            _exists_ok=_exists_ok,
+            _skip_extra=_skip_extra,
+            **self.tg.read())
+
+    def write(self, *, force=False, _exists_ok=False, _skip_extra=False, **items):
         """
         Creates a db file with the core schema.
 
         :param force: If `True` an existing db file will be overwritten.
         """
         if self.fname and self.fname.exists():
-            raise ValueError('db file already exists, use force=True to overwrite')
+            if not force:
+                raise ValueError('db file already exists, use force=True to overwrite')
+            else:
+                self.fname.unlink()
 
         with self.connection() as db:
             for table in self.tables:
@@ -390,9 +444,18 @@ class Database(object):
                             assert pk
                             at = t.many_to_many[k]
                             atkey = tuple([at.name] + [c.name for c in at.columns])
-                            for vv in v:
-                                refs[atkey].append((pk, vv, k))
+                            # We distinguish None - meaning NULL - and [] - meaning no items - as
+                            # values of list-valued columns.
+                            for vv in (v or []):
+                                fkey, context = self.association_table_context(t, k, vv)
+                                refs[atkey].append((pk, fkey, context))
                         else:
+                            if k not in cols:
+                                if _skip_extra:
+                                    continue
+                                else:
+                                    raise ValueError(
+                                        'unspecified column {0} found in data'.format(k))
                             col = cols[k]
                             if isinstance(v, list):
                                 # Note: This assumes list-valued columns are of datatype string!
